#!/usr/bin/env python
# -*- coding: latin-1 -*-
# vim:set ts=4 sw=4 si et sta sts=4 fenc=latin1 :
# Copyright (C) 2009 by Intevation GmbH
# Authors:
# Karl-Heinz Ruskowski <khruskowski@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys, os, time, math, types, odbc, re

import wasko_mapper
from uuid import uuid4

from PyQt4 import Qt, QtCore, QtGui
from threading import Thread
from UI.Wizard import Ui_Wizard

from UI.start import *
from UI.inputfile_open import *
from UI.choosen_operations import *
from UI.select_output import *
from UI.process_files import *
from UI.end import *

import codecs

def map_tagebuch_datum(d):
    res = re.match('(\d+)\.(\d+)\.(\d+)', str(d))

    if res != None:
        date = '%s-%s-%sT12:00:00' % (res.group(3), res.group(2), res.group(1))
        return date
    else:
        return ''

def map_fk(u):
    #print type(u)
    if type(u) == str: u = unicode(u, 'latin-1')
    map = {'1': u'ja', '0': u'nein', '-1': u'keine Angabe', '-3': u'unbekannt'}
    return u'' if u is None else map.get(u, u"unbekannt")

#    if u != None and map.has_key(str(u)):
#        return map[str(u)]
#    else:
#        return 'unbekannt'

order_elements = [
'uuid_id', 'einwilligung', 'phase',
'fkz', 'ks', 'fn',
'mitarb_zv', 'mitarb_zn', 'mitarb_vv',
'mitarb_vn', 'name', 'vorname',
'geschlecht', 'geburtsdatum', 'erstgespraech',
'kontakt', 'zugang', 'zugang_t',
'cm2c', 'cm2c_t', 'kd_schule_name_1',
'kd_schule_strasse_1', 'kd_schule_strassenr_1', 'kd_schule_plz_1',
'kd_schule_ort_1', 'kd_schule_vorwahl_1', 'kd_schule_telefonnummer_1',
'kd_schule_email_1', 'bezugs_name_1', 'bezugs_vorname_1',
'bezugs_vorwahl_1', 'bezugs_telefonnummer_1', 'bezugs_email_1',
'kd_jugendamt_kundennummer_1', 'kd_jugendamt_strasse_1', 'kd_jugendamt_strassenr_1',
'kd_jugendamt_plz_1', 'kd_jugendamt_ort_1', 'kd_jugendamt_vorwahl_1',
'kd_jugendamt_telefonnummer_1', 'kd_jugendamt_email_1', 'kd_institution_name_1',
'kd_institution_kundennummer_1', 'kd_institution_strasse_1', 'kd_institution_strassenr_1',
'kd_institution_plz_1', 'kd_institution_ort_1', 'kd_institution_vorwahl_1',
'kd_institution_telefonnummer_1', 'kd_institution_email_1', 'kd_institution_name_2',
'kd_institution_kundennummer_2', 'kd_institution_strasse_2', 'kd_institution_strassenr_2',
'kd_institution_plz_2', 'kd_institution_ort_2', 'kd_institution_vorwahl_2',
'kd_institution_telefonnummer_2', 'kd_institution_email_2', 'kd_institution_name_3',
'kd_institution_kundennummer_3', 'kd_institution_strasse_3', 'kd_institution_strassenr_3',
'kd_institution_plz_3', 'kd_institution_ort_3', 'kd_institution_vorwahl_3',
'kd_institution_telefonnummer_3', 'kd_institution_email_3', 'dok_eingang',
'datum_cm_start', 'addresse_strasse_1', 'addresse_strassenr_1',
'addresse_plz_1', 'addresse_ort_1', 'addresse1_vorwahl_1',
'addresse1_telefonnummer_1', 'addresse2_vorwahl_1', 'addresse2_telefonnummer_2',
'addresse_weiteretelefonnummern_1', 'addresse_email_1', 'wohnort',
'wohnauf', 'wohnauf_t', 'kinder',
'kinder_hh', 'kinder_allein', 'nat_land', 'nat_land_t', 'nat_staat',
'nat_staat_t', 'nat_status', 'nat_wohn', 'nat_muspra', 'nat_muspra_t',
'nat_haspra', 'psorge', 'psorge_t', 'mutter', 'mutter_n',
'mutter_vn', 'mutter_gb', 'mutter_strasse_1',
'mutter_strassenr_1', 'mutter_plz_1', 'mutter_ort_1',
'mutter_vorwahl1_1', 'mutter_telefonnummer1_1', 'mutter_vorwahl2_1',
'mutter_telefonnummer2_1', 'mutter_email_1', 'mutter_land',
'mutter_land_t', 'mutter_staat', 'mutter_spra',
'mutter_bildung', 'mutter_bildung_t', 'mutter_beruf',
'mutter_sonst', 'vater', 'vater_n',
'vater_vn', 'vater_gb', 'vater_strasse_1',
'vater_strassenr_1', 'vater_plz_1', 'vater_ort_1',
'vater_vorwahl1_1', 'vater_telefonnummer1_1', 'vater_vorwahl2_1',
'vater_telefonnummer2_1', 'vater_email_1', 'vater_land',
'vater_land_t', 'vater_staat', 'vater_spra',
'vater_bildung', 'vater_bildung_t', 'vater_beruf',
'vater_sonst', 'geschw_vorhanden', 'geschw_w',
'geschw_m', 'geschw_hh', 'geschw_alt_a',
'geschw_alt_b', 'geschw_alt_c', 'geschw_alt_d',
'einkommen_a', 'einkommen_b', 'einkommen_c', 'einkommen_e',
'einkommen_d', 'einkommen_d_t', 'bezugsperson_a',
'bezugsperson_b', 'bezugsperson_c', 'bezugsperson_d',
'bezugsperson_e', 'bezugsperson_f', 'bezugsperson_g',
'bezugsperson_h', 'bezugsperson_i', 'bezugsperson_j',
'bezugsperson_k', 'bezugsperson_l', 'bezugsperson_m',
'bezugsperson_n', 'bezugsperson_o', 'bezugsperson_p',
'bezugsperson_q', 'bezugsperson_r', 'bezugsperson_r_t',
'kd_bezug_name_1', 'kd_bezug_vorname_1', 'kd_bezug_funktion_1',
'kd_bezug_strasse_1', 'kd_bezug_strassenr_1', 'kd_bezug_plz_1',
'kd_bezug_ort_1', 'kd_bezug_1_vorwahl', 'kd_bezug_1_telefon',
'kd_bezug_email_1', 'kd_bezug_name_2', 'kd_bezug_vorname_2',
'kd_bezug_funktion_2', 'kd_bezug_strasse_2', 'kd_bezug_strassenr_2',
'kd_bezug_plz_2', 'kd_bezug_ort_2', 'kd_bezug_2_vorwahl',
'kd_bezug_2_telefon', 'kd_bezug_email_2', 'kd_bezug_name_3',
'kd_bezug_vorname_3', 'kd_bezug_funktion_3', 'kd_bezug_strasse_3',
'kd_bezug_strassenr_3', 'kd_bezug_plz_3', 'kd_bezug_ort_3',
'kd_bezug_3_vorwahl', 'kd_bezug_3_telefon', 'kd_bezug_email_3',
'zugeh_a', 'zugeh_b', 'zugeh_c',
'zugeh_d', 'dok_angaben', 'schulform',
'schulform_t', 'klassenstufe', 'klassenwdh',
'sinteresse_a', 'sinteresse_b', 'leistung_ges',
'leistung_d', 'leistung_m', 'leistung_fs',
'sv_aktiv', 'sv_passiv', 'sv_a',
'sv_b', 'sv_c', 'sv_d',
'sv_e', 'sv_f', 'sv_g',
'sv_g_t', 'fehltg_sj', 'fehltg_sj_ue',
'fehltg_m', 'fehltg_m_ue', 'fehlst_sj',
'fehlst_sj_ue', 'fehlst_m', 'fehlst_m_ue',
'sv_monate', 'abschl_gef', 'abschl_pr',
'ordmn', 'ekomp_a', 'ekomp_b',
'ekomp_c', 'ekomp_d', 'ekomp_e',
'ekomp_f', 'ekomp_g', 'ekomp_h',
'ekomp_i', 'ekomp_i_t', 'problfam_a',
'problfam_b', 'problfam_c', 'problfam_d',
'problfam_e', 'problfam_f', 'problfam_g',
'problfam_h', 'problfam_i', 'problfam_j',
'problfam_k', 'problfam_l', 'problfam_m',
'problfam_m_t', 'familie_anm', 'uleistung_beginn',
'uleistung_beginn_a', 'uleistung_beginn_b', 'uleistung_beginn_c',
'uleistung_beginn_d', 'uleistung_beginn_e', 'uleistung_beginn_f',
'uleistung_beginn_g', 'uleistung_beginn_h', 'uleistung_beginn_i',
'uleistung_beginn_j',
'uleistung_beginn_j_t', 'uleistung_verg', 'uleistung_anm',
'dok_biograf', 'datum_kf_1', 'kf_verfahren_1',
'kf_anbiet_1', 'kf_meth_a_1', 'kf_meth_b_1',
'kf_meth_c_1', 'kf_meth_d_1', 'kf_meth_e_1',
'kf_meth_f_1', 'kf_meth_f_t_1', 'kf_test_a_1',
'kf_test_a_t_1', 'kf_test_b_1', 'kf_test_b_t_1',
'kf_test_c_1', 'kf_test_c_t_1', 'kf_test_d_1',
'kf_test_d_t_1', 'komp_schule_a_1', 'komp_schule_b_1',
'komp_schule_c_1', 'komp_schule_d_1', 'komp_schule_e_1',
'komp_schule_f_1', 'komp_schule_g_1', 'komp_schule_g_t_1',
'komp_deutsch_a_1', 'komp_deutsch_b_1', 'komp_deutsch_c_1',
'komp_deutsch_d_1', 'komp_herkunft_1', 'komp_sozial_a_1',
'komp_sozial_b_1', 'komp_sozial_c_1', 'komp_sozial_d_1',
'komp_sozial_e_1', 'komp_sozial_f_1', 'komp_sozial_g_1',
'komp_sozial_g_t_1', 'komp_selbst_a_1', 'komp_selbst_b_1',
'komp_selbst_c_1', 'komp_selbst_d_1', 'komp_selbst_e_1',
'komp_selbst_f_1', 'komp_selbst_g_1', 'komp_selbst_g_t_1',
'komp_psy_a_1', 'komp_psy_b_1', 'komp_psy_c_1',
'komp_psy_d_1', 'komp_psy_e_1', 'komp_psy_f_1',
'komp_psy_g_1', 'komp_psy_g_t_1', 'komp_anm',
'entw_a_1', 'entw_b_1', 'entw_c_1',
'entw_d_1', 'entw_e_1', 'entw_f_1',
'entw_g_1', 'entw_h_1', 'entw_h_t_1',
'entw_i_1', 'entw_i_t_1', 'entw_j_1',
'entw_j_t_1', 'entw_k_1', 'entw_k_t_1',
'entw_beh', 'dok_entw', 'datum_bfplan',
'beschul_a', 'beschul_b', 'beschul_c',
'beschul_d', 'beschul_e', 'beschul_f',
'beschul_h', 'beschul_h_t', 'reint_ziel',
'reint_ziel_t', 'reint_beding_a', 'reint_beding_b',
'reint_beding_c', 'reint_beding_d_t', 'reint_beding_d',
'fehlm_1', 'fehlj_1', 'fehltg_1',
'fehlutg_1', 'fehlst_1', 'fehlfust_1',
'fehlm_2', 'fehlj_2', 'fehltg_2',
'fehlutg_2', 'fehlst_2', 'fehlfust_2',
'fehlm_3', 'fehlj_3', 'fehltg_3',
'fehlutg_3', 'fehlst_3', 'fehlfust_3',
'fehlm_4', 'fehlj_4', 'fehltg_4',
'fehlutg_4', 'fehlst_4', 'fehlfust_4',
'fehlm_5', 'fehlj_5', 'fehltg_5', 'fehlutg_5',
'fehlst_5', 'fehlfust_5', 'fehlm_6',
'fehlj_6', 'fehltg_6', 'fehlutg_6',
'fehlst_6', 'fehlfust_6', 'fehlm_7',
'fehlj_7', 'fehltg_7', 'fehlutg_7',
'fehlst_7', 'fehlfust_7', 'fehlm_8',
'fehlj_8', 'fehltg_8', 'fehlutg_8',
'fehlst_8', 'fehlfust_8', 'fehlm_9',
'fehlj_9', 'fehltg_9', 'fehlutg_9',
'fehlst_9', 'fehlfust_9', 'fehlm_10',
'fehlj_10', 'fehltg_10', 'fehlutg_10',
'fehlst_10', 'fehlfust_10', 'fehlm_11',
'fehlj_11', 'fehltg_11', 'fehlutg_11',
'fehlst_11', 'fehlfust_11', 'fehlm_12',
'fehlj_12', 'fehltg_12', 'fehlutg_12',
'fehlst_12', 'fehlfust_12', 'probleme_a',
'probleme_b', 'probleme_c', 'probleme_d',
'probleme_e', 'probleme_f_t', 'probleme_f',
'zielcm_schul_a', 'zielcm_schul_b', 'zielcm_schul_c',
'zielcm_schul_d', 'zielcm_schul_e', 'zielcm_schul_f',
'zielcm_schul_g', 'zielcm_schul_h', 'zielcm_schul_h_t',
'zielcm_sozial_a', 'zielcm_sozial_b', 'zielcm_sozial_c',
'zielcm_sozial_d', 'zielcm_sozial_e', 'zielcm_sozial_f',
'zielcm_sozial_g', 'zielcm_sozial_h', 'zielcm_sozial_h_t',
'hilfm_1', 'hilfm_art_1', 'hilfm_inst_1',
'hilfm_einr_1', 'hilfm_start_1', 'hilfm_ende_1',
'hilfm_umf_1', 'hilfm_ziel_1_1', 'hilfm_zeit_1_1',
'hilfm_erreicht_1_1', 'hilfm_ziel_1_2', 'hilfm_zeit_1_2',
'hilfm_erreicht_1_2', 'hilfm_ziel_1_3', 'hilfm_zeit_1_3',
'hilfm_erreicht_1_3', 'hilfm_sf_1', 'hilfm_2',
'hilfm_art_2', 'hilfm_inst_2', 'hilfm_einr_2',
'hilfm_start_2', 'hilfm_ende_2', 'hilfm_umf_2',
'hilfm_ziel_2_1', 'hilfm_zeit_2_1', 'hilfm_erreicht_2_1',
'hilfm_ziel_2_2', 'hilfm_zeit_2_2', 'hilfm_erreicht_2_2',
'hilfm_ziel_2_3', 'hilfm_zeit_2_3', 'hilfm_erreicht_2_3',
'hilfm_sf_2', 'hilfm_3', 'hilfm_art_3',
'hilfm_inst_3', 'hilfm_einr_3', 'hilfm_start_3',
'hilfm_ende_3', 'hilfm_umf_3', 'hilfm_ziel_3_1',
'hilfm_zeit_3_1', 'hilfm_erreicht_3_1', 'hilfm_ziel_3_2',
'hilfm_zeit_3_2', 'hilfm_erreicht_3_2', 'hilfm_ziel_3_3',
'hilfm_zeit_3_3', 'hilfm_erreicht_3_3', 'hilfm_sf_3',
'hilfm_weitere', 'vereinb_a', 'vereinb_b',
'vereinb_c', 'vereinb_d', 'vereinb_e',
'vereinb_f_t', 'vereinb_f', 'elt_koop',
'elt_ziel_a', 'elt_zerr_a', 'elt_sf_a',
'elt_ziel_b', 'elt_zerr_b', 'elt_sf_b',
'elt_ziel_c', 'elt_zerr_c', 'elt_sf_c',
'elt_ziel_d', 'elt_zerr_d', 'elt_sf_d',
'elt_ziel_e', 'elt_zerr_e', 'elt_sf_e',
'elt_ziel_f', 'elt_zerr_f', 'elt_sf_f',
'elt_ziel_g', 'elt_zerr_g', 'elt_sf_g',
'elt_ziel_h', 'elt_zerr_h', 'elt_sf_h',
'elt_ziel_i_t', 'elt_ziel_i', 'elt_zerr_i',
'elt_sf_i', 'elt_mn_a', 'elt_mn_b',
'elt_mn_c', 'elt_mn_d', 'elt_mn_e',
'elt_mn_f', 'elt_mn_g', 'elt_mn_h',
'elt_mn_i', 'elt_mn_j', 'elt_mn_j_t',
'dok_cm', 'datum_cm_ende', 'cm_zeit',
'cm_kontakt', 'cm_kontakt_sonstige', 'cm_kontakt_sonstige_t',
'beschul_end_a', 'beschul_end_b', 'beschul_end_c',
'beschul_end_d', 'beschul_end_e', 'beschul_end_f',
'beschul_end_h', 'beschul_end_h_t', 'uleistung_ende',
'uleistung_ende_a', 'uleistung_ende_b', 'uleistung_ende_c',
'uleistung_ende_d', 'uleistung_ende_e', 'uleistung_ende_f',
'uleistung_ende_g', 'uleistung_ende_h', 'uleistung_ende_i',
'uleistung_ende_j', 'uleistung_ende_j_t', 'cm_end_art',
'cm_end_verm', 'cm_end_verm_t', 'cm_end_schul',
'cm_end_schul_t', 'cm_end_jg', 'cm_schule_name',
'cm_schule_strasse', 'cm_schule_strassenr', 'cm_schule_plz',
'cm_schule_ort', 'cm_schule_person_1', 'cm_schule_person_2',
'cm_end_sonst', 'cm_end_sonst_t', 'cm_end_ab',
'cm_end_abgr', 'cm_end_abgr_t', 'cm_ziel',
'cm_schulbes_1', 'cm_schulbes_2', 'cm_zielsch_a',
'cm_zielsch_b', 'cm_zielsch_c', 'cm_zielsch_d',
'cm_zielsch_e', 'cm_zielsch_f', 'cm_zielsch_g',
'cm_zielsch_h', 'cm_zielsch_h_t', 'cm_zielsoz_a',
'cm_zielsoz_b', 'cm_zielsoz_c', 'cm_zielsoz_d',
'cm_zielsoz_e', 'cm_zielsoz_f', 'cm_zielsoz_g',
'cm_zielsoz_h', 'cm_zielsoz_h_t', 'dok_ziel',
'nbtr', 'nbetr_start', 'nbetr_ende',
'nbetr_mn', 'nbetr_verbl', 'nbetr_verbl_t',
'dok_abschl'
]

XML_HEADER = u'<?xml version="1.0" encoding="UTF-8" standalone="yes"?>\n'

from xml.sax.saxutils import quoteattr, escape

class Node(object):

   def __init__(self, name):
       self.name = name
       self.children = []
       self.attributes = {}

   def setAttribute(self, key, value):
       self.attributes[key] = value

   def appendChild(self, child):
       self.children.append(child)
       return child

   def attributes_to_xml(self):
       return u" " + u" ".join([u"%s=%s" % (k, quoteattr(v)) \
           for k, v in self.attributes.iteritems()]) \
           if self.attributes else u""

   def to_xml(self, out):
       out.append(u"<%s%s>" % (self.name, self.attributes_to_xml()))
       for child in self.children:
           child.to_xml(out)
       out.append(u"</%s>" % self.name)

class TextNode(Node):
    def __init__(self, name):
        Node.__init__(self, name)
        self.text = ""

    def to_xml(self, out):
        out.append(escape(self.text))

class Document(object):

    def __init__(self, root_name):
        self.documentElement = Node(root_name)

    def createElement(self, node_name):
        return Node(node_name)

    def createTextNode(self, text):
        if type(text) == str: # XXX: Workaround!
            text = unicode(text.decode('latin-1'))
        tnode = TextNode(None)
        tnode.text = text

        return tnode

    def to_xml(self):
        out = [XML_HEADER]
        self.documentElement.to_xml(out)
        return u"".join(out)

class S2CXmlWriter(object):
    def __init__(self, filename):
        self.filename = filename
        self.xml_doc = Document(u'cases')
        self.top_element = self.xml_doc.documentElement
        self.top_element.setAttribute('version', '3')

        # Create Top element Case (for now we have only one case per file)
        case_element = self.xml_doc.createElement('case')
        self.top_element = self.top_element.appendChild(case_element)
        self.case_element = self.top_element

        master = self.xml_doc.createElement('master')
        self.top_element = self.top_element.appendChild(master)

        self.elements = {}

    # add toplevel child element
    def add_element(self, tag, value):
        if value is not None and value != '':
            self.elements[tag] = value
            return True
        return False

    def prewrite(self, order_by=[]):
        tags = self.elements.keys()

        if len(order_by) > 0:
            tags = order_by

        for tag in tags:
            if self.elements.has_key(tag):
                value = self.elements[tag]

                if value is None: continue

                node_value = None
                element = self.xml_doc.createElement(tag)
                n = self.top_element.appendChild(element)

                value = wasko_mapper.to_unicode(value)

                node_value = self.xml_doc.createTextNode(value)
                n.appendChild(node_value)

    def write(self):
        #self.outfile = open(self.filename, 'w')
        #self.xml_doc.writexml(self.outfile, newl='\n', encoding='UTF-8')
        txt = self.xml_doc.to_xml()
        out = None
        try:
            out = codecs.open(self.filename, "w", "UTF-8")
            print >> out, txt
        finally:
            if out: out.close()

class Tagebuch(object):
    def __init__(self, xmlw):
        self.xmlw = xmlw

    def open(self):
        self.xml_doc = self.xmlw.xml_doc
        self.top_element = self.xmlw.case_element

        e = self.xml_doc.createElement('tagebuch')
        self.tagebuch = self.top_element.appendChild(e)

    def create_entry(self, date, editor, logtype, logmisc, lognotice, logduration):
        # append tagebuch-eintrag to top element
        e = self.xml_doc.createElement('tagebuch-eintrag')

        tagebuch_eintrag = self.tagebuch.appendChild(e)

        uuid_id = str(uuid4())
        uuid = self.xml_doc.createElement('uuid_id')
        uuid_t = self.xml_doc.createTextNode(uuid_id)
        uuid.appendChild(uuid_t)
        tagebuch_eintrag.appendChild(uuid)

        date = wasko_mapper.to_unicode(date).strip()
        if date:
            tagebuch_datum = self.xml_doc.createElement('tagebuch-datum')
            tagebuch_datum_t = self.xml_doc.createTextNode(date)
            tagebuch_datum.appendChild(tagebuch_datum_t)
            tagebuch_eintrag.appendChild(tagebuch_datum)

        editor = wasko_mapper.to_unicode(editor).strip()
        if editor:
            tagebuch_bearbeiter = self.xml_doc.createElement('tagebuch-bearbeiter')
            tagebuch_bearbeiter_t = self.xml_doc.createTextNode(editor)
            tagebuch_bearbeiter.appendChild(tagebuch_bearbeiter_t)
            tagebuch_eintrag.appendChild(tagebuch_bearbeiter)

        logtype = wasko_mapper.to_unicode(logtype).strip()
        if logtype:
            tagebuch_art = self.xml_doc.createElement('tagebuch-art')
            tagebuch_art_t = self.xml_doc.createTextNode(logtype)
            tagebuch_art.appendChild(tagebuch_art_t)
            tagebuch_eintrag.appendChild(tagebuch_art)

        logmisc = wasko_mapper.to_unicode(logmisc).strip()
        if logmisc:
            tagebuch_sonst = self.xml_doc.createElement('tagebuch-sonstiges')
            tagebuch_sonst_t = self.xml_doc.createTextNode(logmisc)
            tagebuch_sonst.appendChild(tagebuch_sonst_t)
            tagebuch_eintrag.appendChild(tagebuch_sonst)

        lognotice = wasko_mapper.to_unicode(lognotice).strip()
        if lognotice:
            tagebuch_notiz = self.xml_doc.createElement('tagebuch-notiz')
            tagebuch_notiz_t = self.xml_doc.createTextNode(lognotice)
            tagebuch_notiz.appendChild(tagebuch_notiz_t)
            tagebuch_eintrag.appendChild(tagebuch_notiz)

        logduration = wasko_mapper.to_unicode(logduration).strip()
        if logduration:
            tagebuch_dauer = self.xml_doc.createElement('tagebuch-dauer')
            tagebuch_dauer_t = self.xml_doc.createTextNode(logduration)
            tagebuch_dauer.appendChild(tagebuch_dauer_t)
            tagebuch_eintrag.appendChild(tagebuch_dauer)

class MDEReader(object):
    """
        Build an interface to an mdb/mde MS Accessfile and
        allow to execute MS SQL queries
    """
    def __init__(self, db_file):
        self.odbcc = odbc.odbc('Driver={Microsoft Access Driver '
                               '(*.mdb)};DBQ=%s' % db_file)
        self.cur = self.odbcc.cursor()

    def execute(self, qry):
        """
           Execute SQL query and apply func (function) to Queryresult.
           Func must return a valid MSSQL_Query_Result type
        """
        self.cur.execute(qry)
        mssql_result = []

        while True:
            row = self.cur.fetchone()
            if not row: break

            cols = []
            for c in row:
                cols.append(c)

            mssql_result.append(cols)

        return mssql_result

    def save_select(self, fields, table, vpf, add=''):
        #print repr(vpf)
        qry_result = []
        sr = []
        for f in fields:
            #print repr(f)
            #if type(f) == str: f = unicode(f, "latin-1")
            if vpf != None:
                # Access '=' operator does not work with NULL
                sr = self.execute("SELECT IIF(%s <> NULL, CStr(%s), NULL) FROM %s WHERE"
                                  " fnr='%s' %s" % (f,f, table, vpf, add))
            else:
                sr = self.execute("SELECT IIF(%s <> NULL, CStr(%s), NULL) FROM %s WHERE"
                                  " %s IS NOT NULL %s" % (f,f, table, f, add))

            if len(sr) >= 1:
                r = []
                for x in sr:
                    u = x[0]
                    if u is None:
                        r.append(None)
                    elif type(u) == str:
                        encoded = False
                        for enc in ('latin-1', 'utf-8'):
                            try:
                                eu = unicode(u, enc)
                                encoded = True
                                break
                            except UnicodeDecodeError:
                                pass
                        if not encoded: eu = u
                        r.append(eu)
                    else:
                        r.append

                qry_result.append(r)
            else:
                qry_result.append([])

        avg_l = 1
        for qr in qry_result:
            if len(qr) > avg_l:
                avg_l = len(qr)

        results = []
        for i in range(0, avg_l):
            results.append([])
            for y in range(0, len(qry_result)):
                if len(qry_result[y]) > i:
                    results[i].append(qry_result[y][i])
                else:
                    results[i].append(None)

        return results

class MdeFileProcessor(Thread):
    def __init__(self, filenames, outputdir):
        Thread.__init__(self)
        self.file_i = -1
        self.filenames = filenames

        self.outputdir = outputdir
        self.outfiles = []
        self.failedfiles = []

        self.progress_max = float(len(self.filenames))
        self.progress = 0.0

    def run(self):
        for mde_input_file in self.filenames:
            mde_input_file = mde_input_file.toLatin1().data()
            self.file_i += 1

            self.progress += 0.1
            #time.sleep(1)
            try:
                mder = MDEReader(mde_input_file)
                # Figure out how many cases are in this file
                fnrs = mder.execute('SELECT fnr, fall, FAL0010, FAL0020 FROM Fall')
                self.progress += 0.2
                x = 0.6 / len(fnrs)
                for fnr, fall, name, vorname in fnrs:
                    if name != None:
                        uuid = str(uuid4())

                        vn = '%s-%s' % (name, vorname)
                        re.subn(re.compile('[\/\\=*\';,}{]'), '', vn)

                        outfile_path = os.path.join(self.outputdir, 'A-%s-%s-%s.xml' % (fall, vn, uuid))
                        self.outfiles.append(outfile_path)

                        self.progress += x/2

                        xmlw = S2CXmlWriter(outfile_path)
                        tagebuch = Tagebuch(xmlw)

                        wasko_mapper.do_mapping(uuid, fnr, fall, mder, xmlw, tagebuch)

                        xmlw.prewrite(order_by=order_elements)

                        self.process_tagebuch(tagebuch, mder, fnr)

                        xmlw.write()
                        #time.sleep(2)
                        self.progress += x/2

            except ValueError:
                sys.stderr.write('Eingabedatei "%s" konnte nicht'
                                 ' gelesen werden.' % mde_input_file.encode('latin-1'))

                raise

            except TypeError:
                del self.outfiles[-1]
                self.failedfiles.append(mde_input_file)
                raise

            self.progress = math.ceil(self.progress)

    def process_tagebuch(self, tagebuch, mder, fnr):
        # Tagebuch
        tagebuch.open()

        falltagebuch = mder.save_select(['datum', 'memo'], 'Falltagebuch', fnr)
        for eintrag in falltagebuch:
            datum = wasko_mapper.extract_all_strings(eintrag[0]).strip()
            notiz = wasko_mapper.extract_all_strings(eintrag[1]).strip()
            if not datum and not notiz: continue
            tagebuch.create_entry(
                map_tagebuch_datum(datum), 'Import', '-2',
                    'Import aus alter Fallakte', notiz, '')

        forderkonferenz = mder.save_select(['FAL2400', 'FAL2410', 'FAL2411', 'FAL2412', 'FAL2413',
                            'FAL2414', 'FAL2416', 'FAL2415', 'FAL2420', 'memo'],
                            'Forderkonferenz', fnr)
        for k in forderkonferenz:
            if k[0] != None:
                konferenz_text = (u'Teilnahme:\n'
                                u'Minderjhrige/r: %s\n'
                                u'Mutter: %s\n'
                                u'Vater: %s\n'
                                u'ASD-Fachkraft: %s\n'
                                u'Lehrer: %s\n'
                                u'Fachkraft der Koordinierungsstelle: %s\n\n'
                                u'Sonstiges:\n%s\n\n'
                                u'Termin der nchsten Helferkonferrenz: %s\n' %
                                (map_fk(k[1]), map_fk(k[2]), map_fk(k[3]),
                                 map_fk(k[4]), map_fk(k[5]), map_fk(k[6]),
                                 map_fk(k[7]), k[8]))
                tagebuch.create_entry(
                    map_tagebuch_datum(k[0]), 'Import', '14',
                    '',  konferenz_text, '')


        pass

    def show_progress(self):
        return self.progress/self.progress_max*100.0

    def show_subject(self):
        return self.filenames[self.file_i]

    def show_outfiles(self):
        return self.outfiles

    def show_failed(self):
        return self.failedfiles

class Wizard(QtGui.QMainWindow):
    def __init__(self, parent=None):
        QtGui.QDialog.__init__(self, parent)

        self.ui = Ui_Wizard()
        self.ui.setupUi(self)

        self.timer = QtCore.QBasicTimer()

        # Select Input files
        self.inputfilelist = []

        # Select output files
        self.outputdir = QtCore.QString(QtCore.QByteArray('C:/'))
        self.outfiles = []
        self.failedfiles = []

        # Process input files
        self.mdefp = None

        # Define order of pages, titles, description and handle
        # wich will be executed when next-button has been pressed
        self.steps_i = -1
        self.steps = [{'module': Ui_Start, 'title': '', 'desc': ''},

                      {'module': Ui_InputFileOpen,
                       'title': 'Dateien w\xe4hlen',
                       'desc': 'W\xe4hlen Sie die Datenbank-Dateien aus, welche Sie'
                               ' umwandeln m\xf6chten.\n',
                       'hdl': self.check_inputfiles },

                      {'module': Ui_SaveOutput,
                       'title': 'Ausgabeverzeichnis festlegen',
                       'desc': 'F\xfcr jede Fallakte in den ausgew\xe4hlten '
                                'Datenbanken wird eine Datei angelegt.\n'
                                'W\xe4hlen Sie ein Verzeichnis aus, in dem die'
                                ' Fallakten abgelegt werden sollen.\n',
                        'hdl': self.check_outputdir },

                      {'module': Ui_ChoosenOperations,
                       'title': 'Gew\xe4hlte Optionen',
                       'desc': '',
                       'hdl': self.do_operations },

                      {'module': Ui_ProcessFiles,
                       'title': 'Dateien umwandeln',
                       'desc': 'Die ausgew\xe4hlten Datenbanken werden '
                                'umgewandelt.',
                       'hdl': self.show_end },

                      {'module': Ui_End, 'title': '', 'desc': ''}]

        # Stack pagewidget into a stacked layout
        sl = QtGui.QStackedLayout()
        for i in self.steps:
            w = QtGui.QWidget(self)
            w.ui = i['module']()
            w.ui.setupUi(w)

            sl.addWidget(w)

        # Add stacked layout to central widget
        self.ui.main_widget.setLayout(sl)

        # Show fist page
        self.show_next()

    def show_next(self):
        # Execute handle
        make_step = True
        if self.steps[self.steps_i].has_key('hdl'):
            make_step = self.steps[self.steps_i]['hdl']()

        # Show next page
        if make_step:
            self.steps_i += 1
            self.ui.main_widget.layout().setCurrentIndex(self.steps_i)
            self.update_nav()

    def show_prev(self):
        # Show privious page
        self.steps_i -= 1
        self.ui.main_widget.layout().setCurrentIndex(self.steps_i)
        self.update_nav()

    def update_nav(self):
        ''' Update wizard navigation'''

        # Set labels
        self.ui.title_label.setText(self.steps[self.steps_i]['title'])
        self.ui.description_label.setText(self.steps[self.steps_i]['desc'])

        # reset nav
        self.ui.next_button.setEnabled(True)
        self.ui.prev_button.setEnabled(True)
        self.ui.cancel_button.setText('Abbrechen')

        # Disable next and prev button on last page
        if self.steps_i == len(self.steps)-1:
            self.ui.prev_button.setEnabled(False)
            self.ui.next_button.setEnabled(False)
            self.ui.cancel_button.setText('Fertigstellen')

        # Disable prev button on first page
        elif self.steps_i == 0:
            self.ui.prev_button.setEnabled(False)

    def check_inputfiles(self):
        '''Check for existing inputfiles, show a warning if there are none. '''

        if len(self.inputfilelist) > 0:
            return True
        else:
            QtGui.QMessageBox.information(self,
                              'Dateien w\xe4hlen',
                              'Sie haben keine Datenbank-Dateien ausgew\xe4hlt.')
            return False

    def check_outputdir(self):
        ''' Check for an output directory and beginn proccessing mde files '''
        ''' warn otherwise. '''

        if self.outputdir == '' and not os.path.exists(self.outputdir):
            QtGui.QMessageBox.information(self,
                                       'Speicherort w\xe4hlen',
                                       'Sie haben kein Ausgabeverzeichnis'
                                       ' gew\xe4hlt oder das Verzeichnis'
                                       ' existiert nicht.')
            return False

        else:
            c_operations = ('Die Fallakten werden gespeichert unter:\n'
                            '%s\n\n'
                            'W\xe4hlen Sie "Weiter", um die Umwandlung '
                            'zu starten. Mit "Zur\xfcck" k\xf6nnen Sie Ihre '
                            'Auswahl \xe4ndern.'
                             % (self.outputdir.toLatin1().data()))

            i = self.ui.main_widget.layout().currentIndex()
            self.ui.main_widget.layout().widget(i+1).ui.output_filelist.addItems(self.inputfilelist)
            self.ui.main_widget.layout().widget(i+1).ui.label_2.setText(c_operations)
            return True

    def do_operations(self):
            self.ui.main_widget.layout().setCurrentIndex(2)

            # Beginn proccessing
            self.timer.start(100, self)
            self.process_files()

            return True

    def show_end(self):
        of = ''
        for x in self.outfiles: of += '%s\n' % x
        t_end = ("Folgende Dateien wurden erstellt:\n"
                "%s\n\n" % of)

        if len(self.failedfiles) > 0:
            ff = ''
            for x in self.failedfiles: ff += '%s\n' % x
            t_end += 'Diese Dateien konnten nicht konvertiert werden:\n%s\n\n' % ff

        t_end += "Diese Dateien koennen nun mit dem  WASKO-Offline-Klienten geoeffnet werden."

        i = self.ui.main_widget.layout().currentIndex()
        self.ui.main_widget.layout().widget(i+1).ui.success_filelist.addItems(self.outfiles)
        self.ui.main_widget.layout().widget(i+1).ui.fail_filelist.addItems(self.failedfiles)
        return True

    def timerEvent(self, event):
        ''' React to timeevent from self.timer. '''
        self.process_files()

    def process_files(self):
        ''' Transform MDE files into our xml format. '''

        # Disable navigation
        self.ui.next_button.setEnabled(False)
        self.ui.prev_button.setEnabled(False)

        # Make sure we see that page that has the progressbar
        # (we will not see it for the first few rounds)
        if self.ui.main_widget.layout().currentIndex() == 4:
            if self.mdefp == None:
                self.mdefp = MdeFileProcessor(self.inputfilelist, 
                                              self.outputdir.toLatin1().data())
                self.mdefp.start()

            else:
                # Update progessbar & status
                p = self.mdefp.show_progress()
                s = self.mdefp.show_subject()
                self.ui.main_widget.layout().currentWidget().ui.mdefile_progressBar.setValue(p)
                self.ui.main_widget.layout().currentWidget().ui.progress_label.setText(s)

                # Quit loop if we are done, move one page ahead
                if self.mdefp.isAlive() == False:
                    self.timer.stop()
                    self.outfiles = self.mdefp.show_outfiles()
                    self.failedfiles = self.mdefp.show_failed()
                    self.show_next()
                    return

if __name__ == '__main__':
    app = QtGui.QApplication(sys.argv)
    w = Wizard()
    w.show()
    sys.exit(app.exec_())

