# -*- coding: latin-1 -*-
# vim:set ts=4 sw=4 si et sta sts=4 fenc=latin1 :
# Copyright (C) 2009 by Intevation GmbH
# Authors:
# Karl-Heinz Ruskowski <khruskowski@intevation.de>
#
# This program is free software under the GPL (>=v2)
# Read the file COPYING coming with the software for details.

import sys, time, re

HN    = re.compile(r"(\d.*)$") # to split Strassenname and Hausnummer
MH    = re.compile(r"^[a-qA-Q]$") # Square in Mannheim
MH_HN = re.compile(r"(\d[^\s,]*)[\s,]*(\d?.*)") # Hausnummer in Mannheim


def NA(x):
    return '-/-' if not x else x

def to_unicode(s):
    if s is None: return u''
    if type(s) == str: 
        for enc in ('latin1', 'utf-8'):
            try:
                return unicode(s, enc)
            except UnicodeDecodeError:
                pass
        return unicode(s, 'latin1', 'replace')
    if type(s) != unicode: return unicode(s)
    return s

# Extract Single Result
def extract_sr(r):
    if len(r) > 0:
        return r[0][0]

    return []

def extract_all_strings(x):
    if x is None: return u''
    if type(x) in (list, tuple):
        out = []
        for y in x:
            if y is not None:
                z = extract_all_strings(y)
                if z: out.append(z)
        return '\n'.join(out)
    if type(x) == str:
        for enc in ('latin1', 'utf-8'):
            try:
                return unicode(x, enc)
            except UnicodeDecodeError:
                pass
        return unicode(s, 'latin1', 'replace')
    if not type(x) == unicode:
        x = unicode(x)
    return x

# Show only the first row
def show_first_row(r):
    if len(r) > 0:
        return r[0]
    else:
        return []

def transform_do(d):
    ''' Transform Date Object '''
    res = re.match('(\d+)\.(\d+)\.(\d+)', str(d))

    if res != None:
        date = '%s-%s-%s' % (res.group(3), res.group(2), res.group(1))
        return date
    else:
        return ''

## XXX: This does not work! (slt)
#def split_str_nr(data, str_i, ort_i):
#    if (data != None and len(data) >= ort_i
#         and data[ort_i] != None
#          and data[ort_i] != 'Mannheim'):
#        res = re.match('^(.*) (\w+)$', str(data[str_i]))
#        if res != None:
#            return (res.group(1), res.group(2))
#        else:
#            return ('', '')
#    else:
#        return ('', '')

def sn_hn(s):
    '''Splits single string containing Strassenname and Hausnummer
       into two strings. Special treatment is applied for the
       Mannheim case.
    '''
    if not s: return '', ''
    hn = HN.search(s)
    hn = hn and hn.group(1) or ''
    sn = s[:len(s)-len(hn)].strip()
    hn = hn.strip()
    if MH.match(sn):
        hn2 = MH_HN.search(hn)
        if hn2: return '%s %s' % (sn, hn2.group(1)), hn2.group(2)
    return sn, hn 

def map_wohnauf(w):
    map = {'1': 1, '2': 2, '3': 3, '4': 4, '5': 5,
           '6': 6, '7': 7, '8': 8, '9': 9, '10': -2}

    if w != None and map.has_key(w):
        return map[w]
    else:
        return '-1'

def map_nat_land(l):
    map = { 'AFG': 0, 'AL': 1, 'DZ': 2, 'AND': 3, 'ANG': 4,
            'AG': 5, 'RA': 6, 'ARM': 7, 'AZ': 8, 'AUS': 9,
            'BS': 10, 'BRN': 11, 'BD': 12, 'BDS': 13, 'B': 14,
            'BH': 15, 'DY': 16, 'BHT': 17, 'BOL': 18, 'BIH': 19,
            'RB': 20, 'BR': 21, 'BRU': 22, 'BG': 23, 'BF': 24,
            'BU': 25, 'RCH': 26, 'VRC': 27, 'CR': 28, 'D': 29,
            'WD': 30, 'DOM': 31, 'DJI': 32, 'DK': 33, 'EC': 34,
            'ES': 35, 'CI': 36, 'ER': 37, 'EST': 38, 'FJI': 39,
            'FIN': 40, 'F': 41, 'G': 42, 'WAG': 43, 'GE': 44,
            'GH': 45, 'WG': 46, 'GR': 47, 'GB': 48, 'GCA': 49, 'RG': 50,
            'GNB': 51, 'GUY': 52, 'RH': 53, 'HN': 54, 'IND': 55, 'RI': 56,
            'IRQ': 57, 'IR': 58, 'IRL': 59, 'IS': 60, 'IL': 61, 'I': 62,
            'JA': 63, 'J': 64, 'YE': 65, 'JOR': 66, 'K': 67, 'RFC': 68,
            'CDN': 69, 'CV': 70, 'KZ': 71, 'Q': 72, 'EAK': 73, 'KS': 74,
            'KIR': 75, 'CO': 76, 'COM': 77, 'ZR': 78, 'RCB': 79, 'DVRM': 80,
            'ROK': 81, 'HR': 82, 'C': 83, 'KT': 84, 'LAO': 85, 'LS': 86,
            'LV': 87, 'RL': 88, 'LB': 89, 'LAR': 90, 'FL': 91, 'LT': 92,
            'L': 93, 'RM': 94, 'MW': 95, 'MAL': 96, 'MV': 97, 'RMM': 98,
            'M': 99, 'MA': 100, 'MH': 101, 'RIM': 102, 'MS': 103, 'MK': 104,
            'MEX': 105, 'FSM': 106, 'MC': 108, 'MNG': 109, 'MOC': 111,
            'MYA': 112, 'NAM': 113, 'NAU': 114, 'NEP': 115, 'NZ': 116,
            'NIC': 117, 'NL': 118, 'RN': 119, 'WAN': 120, 'N': 121,
            'OM': 122, 'PK': 124, 'PAL': 125, 'PA': 126, 'PNG': 127,
            'PY': 128, 'PE': 129, 'RP': 130, 'PL': 131, 'P': 132,
            'RWA': 133, 'RO': 134, 'RUS': 135, 'SOL': 136, 'Z': 137,
            'WS': 138, 'RSM': 139, 'KSA': 140, 'S': 141, 'CH': 142,
            'SN': 143, 'SY': 145, 'WAL': 146, 'ZW': 147, 'SGP': 148,
            'SK': 149, 'SLO': 150, 'SP': 151, 'E': 152, 'CL': 153,
            'KN': 154, 'WL': 155, 'WV': 156, 'SUD': 157, 'SME': 158,
            'SD': 159, 'SYR': 160, 'STP': 161, 'RSA': 162, 'TJ': 163,
            'EAT': 164, 'THA': 165, 'RT': 166, 'TO': 167, 'TT': 168,
            'TCH': 169, 'CZ': 170, 'TN': 171, 'TM': 172, 'TUV': 173,
            'TR': 174, 'EAV': 175, 'UA': 176, 'H': 177, 'ROU': 178,
            'UZB': 179, 'VU': 180, 'V': 181, 'YU': 182, 'UAE': 183,
            'USA': 184, 'VN': 185, 'BY': 186, 'RCA': 187, 'CY': 188,
            'ET': 189, 'GQ': 190, 'ETH': 191, 'A': 192 }

    if l != None and map.has_key(l[:-1]):
        return map[l[:-1]]
    elif l != None:
        return '-2'
    else:
        return '-1'

def map_nat_staat(s):
    map = {'deutsch': 1, 'andere': 2}
    if s != None and map.has_key(s):
        x = map[s]
    else:
        x = '-1'
    return x

def map_uleistung(u):
    map = {'ja': 1, 'nein': 0, 'keine angabe': -1, 'unbekannt': -3}
    if u != None and map.has_key(u):
        return map[u]
    else:
        return '-1'

def map_geschlecht(g):
    map = {'m': 0, 'w': 1}
    if g != None and map.has_key(g):
        return map[g]
    else:
        return -1

def map_fk(u):
    map = {'1': 'ja', '0': 'nein', '-1': 'keine Angabe', '-3': 'unbekannt', 'None': ''}
    if u != None and map.has_key(str(u)):
        return map[str(u)]
    else:
        return 'keine Angabe'

def map_eltern(e):
    map = {'keine Angabe': -1, 'bekannt': 1, 'nicht bekannt': 2, 'verstorben': 3}
    if e != None and map.has_key(e):
        return map[e]
    else:
        return -1

# TODO: Diese Funktion ist noch einmal zu ueberpruefen: Sie wird weiter 
# mehrfach eingesetzt, aber unklar, ob das sinnvoll ist. Die korrekten Listen
# scheinen gender mainstreamed, was mehr Unterscheidungen implizieren wuerde.
def map_eltern_arbeit(a):
    map = {'unbekannt': -3, 'keine angabe': -1, 'arbeitslos': 1,
        'nicht erwerbst\xe4tig/Hausfrau': 2, 'teilzeit besch\xe4ftigt': 3,
        'geringf\xfegig besch\xe4ftigt/Aushilfe': 4, 'voll besch\xe4ftigt': 5,
        'Rente/Pension': 6}
    if a is not None: return map.get(a, -1)
    return -1
# Nicht relevant, da kein sinnvolles Mapping moeglich, aber
#     map = {'ohne Besch\xe4ftigung', 
#             'geringf\xfegige Besch\xe4ftigung/Aushilfst\xe4tigkeit',
#             'Facharbeiterin',
#             'k\xfenstlerische T\xe4tigkeit',
#             'akademischer Beruf',
#             'selbst\xe4ndige T\xe4tigkeit\Unternehmerin'
#             'Rentnerin/Pension\xe4rin': 6, 
#             'Sonstiges'}
# 
# waere die richtige Liste (fuer die Mutter)). 

def map_kd_bezug(b):
    map = {'sonstiges': -2, 'keine Angabe': -1, 'Mutter': 1, 'Vater': 2,
           'Gro\xdfeltern': 3, 'Geschwister': 4, 'Pflegefamilie': 5,
           'Stiefmutter': 6, 'Stiefvater': 7, 'Verwandte': 8, 'Partnerin/Partner': 9,
           'Betreuerin/Betreuer im Heim/Wohnform': 10, 'gesetzl. bestimmte/r Betreuerin/Bereuer': 11,
           'Lehrerin/Lehrer': 12, 'Mitbewohnerin/Mitbewohner': 13, 'Lebensgef\xe4hrte der Mutter': 14,
           'beste Freundin/bester Freund': 16, 'keine Bezugsperson': 17}
    if b is not None: return map.get(b, -1)
    return '-1'

def map_problfam(p):
    map = { '9': '-3', '99': '-1', '1': '1', '2': '2', '3': '3', '4': '4',
            '5': '5', '6': '6'}
    if p != None and map.has_key(p):
        return map[p]
    else:
        return '-1'

def map_zugeh(p):
    map = { '99': -1, '1': 1, '2': 2, '3': 3, '4': 4,
            '5': 5, '6': 6}
    if p != None and map.has_key(p):
        return map[p]
    else:
        return '-1'

def map_ziel_er(z):
    map = { '9': '-3', '99': '-1', '1': '1', '2': '2', '3': '3', '4': 4,
            '5': '5', '6': '6'}
    if z != None and map.has_key(z):
        return map[z]
    else:
        return '-1'

def map_zielsch(z):
    map = { '9': '-3', '99': '-1', '1': '5', '2': '5', '3': '4', '4': '4',
            '5': '3', '6': '3'}
    if z != None and map.has_key(z):
        return map[z]
    else:
        return '-1'

def map_ziel_er_t(z):
    map = { '9': 'unbekannt', '99': 'keine Angabe', '1': 'Voll erreicht',
            '2': 'Erreicht', '3': 'Eher erreicht', '4': 'Eher nicht erreicht',
            '5': 'Nicht erreicht', '6': 'Gar nicht erreicht'}
    if z != None and map.has_key(z):
        return map[z]
    else:
        return 'keine Angabe'

def map_reint_ziel(z):
    map = {'Unbekannt': -3, 'Sonstiges': -2, 'keine Angabe': -1, 'in die Ursprungsklasse': 1,
           'In die gleiche Schule, aber andere Klasse': 2, 'in eine andere Schule': 3,
           'Praxisklasse': 4}

    if z != None and map.has_key(z):
        return map[z]
    else:
        return -1

def map_hilfm(m):
    map = {'keine Angabe': -1, 'nein': 0, 'ja': 1}
    if m != None and map.has_key(m):
        return map[m]
    else:
        return -1

def map_bildung(b):
    map = {'1': '1', '2': '-3', '3': '6', '4': '7', '5': '7', '6': '-2'}
    if b != None and map.has_key(b):
        return map[b]
    else:
        return '-1'

def map_cm_end_art(m):
    map = {'Reintegration in die Regelschule': '1', 'Abbruch der Ma\xdfnahme durch den jungen Menschen': '3',
           'Umzug': '2', 'Sonstiges': '2'}
    if m != None and map.has_key(m):
        return map[m]
    else:
        return '-1'

def map_cm_end_verm(m):
    map = {'Ursprungsklasse': '1', 'gleiche Schule, neue Klasse': '2',
           'andere Schule': '3', 'Sonstiges': '-2'}
    if m != None and map.has_key(m):
        return map[m]
    else:
        return '-1'

def mk_utime(t):
    res = re.match('(\d+)\.(\d+)\.(\d+)', str(t))
    if res != None:
        return time.mktime((int(res.group(3)), int(res.group(2)),
                            int(res.group(1)), 0, 0, 0, 0, 0, -1))
    else:
        return 0.0

FALL_BEGLEITUNG_A_SCHUL = {
    u'ausserschulischer Lernort'                          : 'c',
    u'innerhalb der Herkunftsschule in einer extra Klasse': 'b',
    u'in der Herkunftsklasse'                             : 'a',
    u'in einer anderen Schule'                            : 'e', 
    u'Praktikum/Betrieb'                                  : 'f',
    u'Sonstiges'                                          : 'h'
}

FALL_BEGLEITUNG_B_SCHUL = {
    u'in die Ursprungsklasse'                   : 'a',
    u'in die gleiche Schule, aber andere Klasse': 'b',
    u'in eine andere Schule'                    : 'e',
    u'Sonstiges'                                : 'h'
}

FALL_BILDUNG_FOERDER = {
    u'in der abgebenden Schule, in der Ursprungsklasse'             : 'a',
    u'in der abgebenden Schule, in einer gesonderten Lerngruppe'    : 'b',
    u'an einem auerschulischen Lernort'                            : 'c',
    u'gem eines Phasenmodells auerhalb und innerhalb der Schule' : 'd' }

FALL_BILDUNG_SCHUL = {
    u'in die Ursprungsklasse':                    'a',
    u'in die gleiche Schule, aber andere Klasse': 'b',
    u'in eine andere Schule':                     'e',
    u'Sonstiges':                                 'h'
}

def beschul_end_mapping(vpf, mder, xml):

    beschul_end = dict(
        [(k, None) for k in ['a', 'b', 'c', 'd', 'e', 'f', 'h', 'h_t']])

    def enlarge_sonstiges(txt):
        txt = txt.strip()
        beschul_end['h'] = '1'
        if txt:
            old = beschul_end['h_t']
            if old is not None:
                beschul_end['h_t'] = u"%s\n%s" % (old, txt)
            else:
                beschul_end['h_t'] = txt

    # look into Fallakte section 'B Bildungs- Foerderplan'
    #bildung = mder.save_select(['FAL2030', 'FAL2035'], 'Bildungsplan', vpf)
    bildung = mder.save_select(['FAL2010'], 'Bildungsplan', vpf)
    if bildung:
        bildung = show_first_row(bildung)
        schul   = extract_all_strings(bildung[0])
        if schul:
            try:
                k = FALL_BILDUNG_FOERDER[schul]
                beschul_end[k] = '1'
            except KeyError:
                pass

    # Handle the 'Fallbegleitungen'
    fallbegleitungen = mder.save_select(['FAL3010', 'FAL3580', 'FAL3015', 'FAL3585'], 'Fallbegleitung', vpf)
    for fallbegleitung in fallbegleitungen:
        a_schul   = extract_all_strings(fallbegleitung[0])
        b_schul   = extract_all_strings(fallbegleitung[1])
        a_schul_s = extract_all_strings(fallbegleitung[2])
        b_schul_s = extract_all_strings(fallbegleitung[3])

        if a_schul:
            try:
                k = FALL_BEGLEITUNG_A_SCHUL[a_schul]
                if k == 'h': enlarge_sonstiges(a_schul_s)
                else:        beschul_end[k] = '1'
            except KeyError:
                enlarge_sonstiges(a_schul_s)

        if b_schul:
            try:
                k = FALL_BEGLEITUNG_B_SCHUL[b_schul]
                if k == 'h': enlarge_sonstiges(b_schul_s)
                else:        beschul_end[k] = '1'
            except KeyError:
                enlarge_sonstiges(b_schul_s)

    # create the XML nodes
    for k, v in beschul_end.iteritems():
        if v: xml.add_element('beschul_end_%s' % k, v)


def do_mapping(uuid, vpf, fallnummer, mder, xmlw, tagebuch):
    dok_cm_5_4 = ''
    dok_angaben_3_7 = ''
    dok_entw_5_1 = u''

    # UUID
    xmlw.add_element('uuid_id', uuid)

    # Einwilligung
    xmlw.add_element('einwilligung', '-1') # (unklar/liegt nicht vor)

    # KS
    tnr = mder.execute('SELECT tnr FROM K_Stelle')
    if len(tnr) > 0:
        ks = mder.execute(
                'SELECT Tr\xe4gername FROM Tr\xe4ger WHERE tnr = %s' % extract_sr(tnr))
    else:
        ks = []
    xmlw.add_element('ks', extract_sr(ks))

    # FN
    xmlw.add_element('fn', 'A-' + str(fallnummer))

    # Name, Vorname, Geschlecht ...
    pers_data = mder.save_select(['FAL0010', 'FAL0020', 'FAL0040', 'FAL0030',
                                  'FAL0140', 'FAL4050'], 'Fall', vpf)
    pers_data = show_first_row(pers_data)
    xmlw.add_element('name', pers_data[0])
    xmlw.add_element('vorname', pers_data[1])
    xmlw.add_element('geschlecht', map_geschlecht(pers_data[2]))
    xmlw.add_element('geburtsdatum', transform_do(pers_data[3]))
    xmlw.add_element('datum_cm_start', transform_do(pers_data[4]))
    xmlw.add_element('datum_cm_ende', transform_do(pers_data[5]))

    xmlw.add_element('zugang', '-1') # Keine Angabe

    xmlw.add_element('cm2c', '-1') # Keine angabe

    # Schule, Bezug
    schule = mder.save_select(['FAL1010', 'FAL1020', 'FAL1030', 'FAL1040', 'FAL1050',
                               'FAL1060', 'FAL1070', 'FAL1080', 'FAL1090', 'FAL1100',
                               'FAL1110'], 'Kompetenzen', vpf)
    schule = show_first_row(schule)
    xmlw.add_element('kd_schule_name_1', schule[0])
    kd_snhn = sn_hn(schule[1])
    #xmlw.add_element('kd_schule_strasse_1', split_str_nr(schule, 1, 3)[0])
    #xmlw.add_element('kd_schule_strassenr_1', split_str_nr(schule, 1, 3)[1])
    xmlw.add_element('kd_schule_strasse_1',   kd_snhn[0])
    xmlw.add_element('kd_schule_strassenr_1', kd_snhn[1])
    xmlw.add_element('kd_schule_plz_1', schule[2])
    xmlw.add_element('kd_schule_ort_1', schule[3])
    xmlw.add_element('kd_schule_vorwahl_1', schule[4])
    xmlw.add_element('kd_schule_telefonnummer_1', schule[5])
    xmlw.add_element('kd_schule_email_1', schule[6])
    xmlw.add_element('bezugs_name_1', schule[7])
    xmlw.add_element('bezugs_vorwahl_1', schule[8])
    xmlw.add_element('bezugs_telefonnummer_1', schule[9])
    xmlw.add_element('bezugs_email_1', schule[10])

    # Eingangs doku
    eing_doku = mder.save_select(['memo1'], 'Fall', vpf)
    xmlw.add_element('dok_eingang', extract_all_strings(eing_doku))

    # Pers. Adresse, wohnauf, nat.
    pers_data = mder.save_select(['FAL0050', 'FAL0060', 'FAL0070', 'FAL0075',
                                  'FAL0080', 'FAL0085', 'FAL0090', 'FAL0100', 'FAL0150',
                                  'FAL0155', 'FAL0110', 'FAL0120', 'FAL0130'],
                                  'Fall', vpf)
    pers_data = show_first_row(pers_data)
    a_snhn = sn_hn(pers_data[0])
    #xmlw.add_element('addresse_strasse_1', split_str_nr(pers_data, 0, 2)[0])
    #xmlw.add_element('addresse_strassenr_1', split_str_nr(pers_data, 0, 2)[1])
    xmlw.add_element('addresse_strasse_1',   a_snhn[0])
    xmlw.add_element('addresse_strassenr_1', a_snhn[1])
    xmlw.add_element('addresse_plz_1', pers_data[1])
    xmlw.add_element('addresse_ort_1', pers_data[2])
    xmlw.add_element('addresse1_vorwahl_1', pers_data[3])
    xmlw.add_element('addresse1_telefonnummer_1', pers_data[4])
    xmlw.add_element('addresse2_vorwahl_1', pers_data[5])
    xmlw.add_element('addresse2_telefonnummer_2', pers_data[6])
    xmlw.add_element('addresse_email_1', pers_data[7])
    xmlw.add_element('wohnort', '-1')
    xmlw.add_element('wohnauf', map_wohnauf(pers_data[8])) # FAL0150
    xmlw.add_element('wohnauf_t', pers_data[9]) # FAL0155
    xmlw.add_element('nat_land', map_nat_land(pers_data[10]))
    xmlw.add_element('nat_staat', map_nat_staat(pers_data[11]))
    #xmlw.add_element('nat_status', '-3')

    if transform_do(pers_data[12]) != '':
        xmlw.add_element('nat_wohn', transform_do(pers_data[12]))

    xmlw.add_element('nat_muspra', '-1')

    # Sorgeberechtigung
    sorge = mder.save_select(['FAL0930', 'FAL0932', 'FAL0931', 'FAL0935'], 'Fall', vpf)
    sorge = show_first_row(sorge)

    s = '-3'
    if sorge[0] == '-1':
        s = '2'
    if sorge[2] == '-1':
        s = '3'
    if sorge[0] == '-1' and sorge[2] == '-1':
        s = '1'
    if sorge[1] == '-1':
        s = '4'

    xmlw.add_element('psorge', s)
    xmlw.add_element('psorge_t', extract_all_strings(sorge[3]))

    # Mutter
    mutter = mder.save_select(['FAL0159', 'FAL0160', 'FAL0170', 'FAL0240', 'FAL0180',
                               'FAL0190', 'FAL0200', 'FAL0205', 'FAL0210', 'FAL0215',
                               'FAL0220', 'FAL0230', 'FAL0250', 'FAL0260', 'FAL0290',
                               'FAL0300', 'FAL0310', 'FAL0320', 'memo2'], 'Fall', vpf)
    mutter = show_first_row(mutter)
    xmlw.add_element('mutter', map_eltern(mutter[0]))
    xmlw.add_element('mutter_n', mutter[1])
    xmlw.add_element('mutter_vn', mutter[2])
    xmlw.add_element('mutter_gb', transform_do(mutter[3]))
    m_snhn = sn_hn(mutter[4])
    #xmlw.add_element('mutter_strasse_1', split_str_nr(mutter, 4, 6)[0])
    #xmlw.add_element('mutter_strassenr_1', split_str_nr(mutter, 4, 6)[1])
    xmlw.add_element('mutter_strasse_1',   m_snhn[0])
    xmlw.add_element('mutter_strassenr_1', m_snhn[1])
    xmlw.add_element('mutter_plz_1', mutter[5])
    xmlw.add_element('mutter_ort_1', mutter[6])
    xmlw.add_element('mutter_vorwahl1_1', mutter[7])
    xmlw.add_element('mutter_telefonnummer1_1', mutter[8])
    xmlw.add_element('mutter_vorwahl2_1', mutter[9])
    xmlw.add_element('mutter_telefonnummer2_1', mutter[10])
    xmlw.add_element('mutter_email_1', mutter[11])
    xmlw.add_element('mutter_land', map_nat_land(mutter[12]))
    xmlw.add_element('mutter_staat', map_nat_staat(mutter[13]))
    xmlw.add_element('mutter_spra', mutter[14])
    xmlw.add_element('mutter_bildung', map_bildung(mutter[15]))
    xmlw.add_element('mutter_bildung_t', mutter[16])
    xmlw.add_element('mutter_beruf', map_eltern_arbeit(mutter[17]))
    xmlw.add_element('mutter_sonst', mutter[18])

    # Vater
    vater = mder.save_select(['FAL0325', 'FAL0330', 'FAL0340', 'FAL0410', 'FAL0350',
                              'FAL0360', 'FAL0370', 'FAL0375', 'FAL0380', 'FAL0385', 'FAL0390',
                              'FAL0400', 'FAL0420', 'FAL0430', 'FAL0460', 'FAL0470',
                              'FAL0480', 'FAL0490', 'memo3'], 'Fall', vpf)
    vater = show_first_row(vater)
    xmlw.add_element('vater', map_eltern(vater[0]))
    xmlw.add_element('vater_n', vater[1])
    xmlw.add_element('vater_vn', vater[2])
    xmlw.add_element('vater_gb', transform_do(vater[3]))
    v_snhn = sn_hn(vater[4])
    #xmlw.add_element('vater_strasse_1', split_str_nr(vater, 4, 6)[0])
    #xmlw.add_element('vater_strassenr_1', split_str_nr(vater, 4, 6)[1])
    xmlw.add_element('vater_strasse_1',   v_snhn[0])
    xmlw.add_element('vater_strassenr_1', v_snhn[1])
    xmlw.add_element('vater_plz_1', vater[5])
    xmlw.add_element('vater_ort_1', vater[6])
    xmlw.add_element('vater_vorwahl1_1', vater[7])
    xmlw.add_element('vater_telefonnummer1_1', vater[8])
    xmlw.add_element('vater_vorwahl2_1', vater[9])
    xmlw.add_element('vater_telefonnummer2_1', vater[10])
    xmlw.add_element('vater_email_1', vater[11])
    xmlw.add_element('vater_land', map_nat_land(vater[12]))
    xmlw.add_element('vater_staat', map_nat_staat(vater[13]))
    xmlw.add_element('vater_spra', vater[14])
    xmlw.add_element('vater_bildung', map_bildung(vater[15]))
    xmlw.add_element('vater_bildung_t', vater[16])
    xmlw.add_element('vater_beruf', map_eltern_arbeit(vater[17]))
    xmlw.add_element('vater_sonst', vater[18])

    # Geschwister
    geschw = mder.save_select(['FAL0500', 'FAL0520', 'FAL0530', 'FAL0540', 'FAL0550', 'FAL0560', 'FAL0570', 'memo4'],
                               'Fall', vpf)
    geschw = show_first_row(geschw)
    if geschw[0] > 0:
        xmlw.add_element('geschw_vorhanden', '1')
    else:
        xmlw.add_element('geschw_vorhanden', '-1')

    try:
        geschw_gesamt = int(geschw[0]) + 0
        geschw_w = int(geschw[1]) + 0

        xmlw.add_element('geschw_w', geschw_w)
        xmlw.add_element('geschw_m', str(geschw_gesamt - geschw_w))

    except:
        pass

    xmlw.add_element('geschw_hh', geschw[6])
    xmlw.add_element('geschw_alt_a', geschw[2])
    xmlw.add_element('geschw_alt_b', geschw[3])
    xmlw.add_element('geschw_alt_c', geschw[4])
    xmlw.add_element('geschw_alt_d', geschw[5])

    geschwister_notiz = extract_all_strings(geschw[7])
    if geschwister_notiz:
        dok_angaben_3_7 += u'Geschwister:\n%s\n\n' % geschwister_notiz

    # Einkommen
    einkomm_a = mder.save_select(['FAL0666', 'FAL0667', 'FAL0668', 'FAL0669', 'FAL0670', 'FAL0671',
                                  'FAL0673', 'FAL0674', 'FAL0672', 'FAL0675', 'FAL0680', 'memo5'],
                                  'Fall', vpf)
    einkomm_a = show_first_row(einkomm_a)


    # Erwerbstaetig 
    if einkomm_a[0] == '-1' or einkomm_a[2] == '-1':
        xmlw.add_element('einkommen_a', '1')
    else:
        xmlw.add_element('einkommen_a', '-1')

    #Geringfuegig 
    if einkomm_a[1] == '-1':
        xmlw.add_element('einkommen_e', '1')
    else:
        xmlw.add_element('einkommen_e', '-1')

    # Unterhaltszahlungen, Elterngeld, Kindergeld
    if einkomm_a[6] == '-1' or einkomm_a[7] == '-1':
        xmlw.add_element('einkommen_c', '1')
    else:
        xmlw.add_element('einkommen_c', '-1')

    # Sozialhilfe
    if einkomm_a[3] == '-1' or einkomm_a[4] == '-1' or einkomm_a[5] == '-1':
        xmlw.add_element('einkommen_b', '1')
    else:
        xmlw.add_element('einkommen_b', '-1')

    # Sonstiges
    if einkomm_a[8] == '-1' or extract_all_strings(einkomm_a[9]) != '':
        xmlw.add_element('einkommen_d', '1')

        ls = []
        if einkomm_a[8] == '-1':
            ls.append("Krankengeld: ja")

        if extract_all_strings(einkomm_a[9]) != '':
            ls.append(extract_all_strings(einkomm_a[9]))

        einkomm_sonst = u"Sonstiges: %s\n" % '; '.join(ls)
    else:
        xmlw.add_element('einkommen_d', '-1')
        einkomm_sonst = u''

    if einkomm_a[10] != None and einkomm_a[10] != '':
        einkomm_sonst += 'Monatliches Nettoeinkommen: %s; ' % extract_all_strings(einkomm_a[10])
    if einkomm_a[11] != None and einkomm_a[11] != '':
        einkomm_sonst += 'Dokumentation: %s;' % extract_all_strings(einkomm_a[11])

    xmlw.add_element('einkommen_d_t', einkomm_sonst)

    # Bezugspersonen
    ebezug = mder.save_select(['Vorname', 'Nachname', 'Funktion', 'Strae', 'PLZ',
                               'Ort', 'Vorwahl', 'Telefon', 'Email', 'Anmerkungen'], 'eBezugsperson', vpf)
    gbezug = mder.save_select(['Vorname', 'Nachname', 'Funktion', 'Strae', 'PLZ',
                               'Ort', 'Vorwahl', 'Telefon', 'Email', 'Anmerkungen'], 'gBezugsperson', vpf)

    # TODO: Issue 2.3.5: Pruefen, ob gleichaltrige Bezugspersonen so
    # tatsaechlich uebernommen werden koennen.
    ebezug.extend(gbezug)

    bezug_n = min(len(ebezug),3)

    notizen_bezugs_personen = []

    for bez in ebezug + gbezug:
        name    = extract_all_strings(bez[1])
        vorname = extract_all_strings(bez[0])
        notiz   = extract_all_strings(bez[9])
        if name or vorname or notiz:
            notizen_bezugs_personen.append(
                u'Notizen zur Bezugsperson %s, %s:' % (NA(name), NA(vorname)))
            notizen_bezugs_personen.append(NA(notiz))

    notizen_bezugs_personen = "\n".join(notizen_bezugs_personen)

    if notizen_bezugs_personen:
        dok_angaben_3_7 += notizen_bezugs_personen

    for e in range(bezug_n):
        i = e+1
        xmlw.add_element('kd_bezug_name_%d' % i, ebezug[e][1])
        xmlw.add_element('kd_bezug_vorname_%d' % i, ebezug[e][0])
        xmlw.add_element('kd_bezug_funktion_%d' % i, map_kd_bezug(ebezug[e][2]))
        e_snhn = sn_hn(ebezug[e][3])
        #xmlw.add_element('kd_bezug_strasse_%d' % i, split_str_nr(ebezug[e], 3, 5)[0])
        #xmlw.add_element('kd_bezug_strassenr_%d' % i, split_str_nr(ebezug[e], 3, 5)[1])
        xmlw.add_element('kd_bezug_strasse_%d' % i,   e_snhn[0])
        xmlw.add_element('kd_bezug_strassenr_%d' % i, e_snhn[1])
        xmlw.add_element('kd_bezug_plz_%d' % i, ebezug[e][4])
        xmlw.add_element('kd_bezug_ort_%d' % i, ebezug[e][5])
        xmlw.add_element('kd_bezug_%d_vorwahl' % i, ebezug[e][6])
        xmlw.add_element('kd_bezug_%d_telefon' % i, ebezug[e][7])
        xmlw.add_element('kd_bezug_email_%d' % i, ebezug[e][8])

    #for bezug in ebezug[bezug_n:]:
    #    dok_angaben_3_7 += 'Bezugsperson (%s, %s):\n%s\n\n' % (
    #        NA(bezug[0]), 
    #        NA(bezug[1]), 
    #        NA(bezug[9]))

    # Kompetenzen
    komp = mder.save_select(['FAL1120', 'FAL1130', 'FAL1400', 'FAL1410', 'FAL1330', 'FAL1340',
                             'FAL1350'], 'Kompetenzen', vpf)

    komp = show_first_row(komp)
    xmlw.add_element('klassenstufe', komp[0])
    xmlw.add_element('klassenwdh', komp[1])
    xmlw.add_element('sinteresse_a', komp[2])
    xmlw.add_element('sinteresse_b', komp[3])
    xmlw.add_element('leistung_ges', komp[4])
    xmlw.add_element('leistung_d', komp[5])
    xmlw.add_element('leistung_m', komp[6])

    komp = mder.save_select(['FAL1150', 'FAL1140', 'FAL1160'],
                             'Kompetenzen', vpf)
    komp = show_first_row(komp)
    xmlw.add_element('fehltg_sj_ue', komp[1])
    xmlw.add_element('fehlst_m_ue', komp[0])
    xmlw.add_element('sv_monate', komp[2])

    komp = mder.save_select(['FAL1650', 'FAL1670', 'FAL1640', 'FAL1630', 'FAL1600', 'FAL1610',
                             'FAL1620', 'FAL1660', 'FAL1785'], 'Kompetenzen', vpf)
    komp = show_first_row(komp)
    xmlw.add_element('ekomp_a', map_problfam(komp[0]))
    xmlw.add_element('ekomp_b', map_problfam(komp[1]))
    xmlw.add_element('ekomp_c', map_problfam(komp[2]))
    xmlw.add_element('ekomp_d', map_problfam(komp[3]))
    xmlw.add_element('ekomp_e', map_problfam(komp[4]))
    xmlw.add_element('ekomp_f', map_problfam(komp[5]))
    xmlw.add_element('ekomp_g', map_problfam(komp[6]))
    xmlw.add_element('ekomp_h', map_problfam(komp[7]))
    xmlw.add_element('ekomp_i', map_problfam(komp[8]))

    komp = mder.save_select(['FAL1680', 'FAL1690', 'FAL1700', 'FAL1710', 'FAL1720', 'FAL1730', 'FAL1740',
                             'FAL1750', 'FAL1760', 'FAL1770', 'FAL1780', 'FAL1786', 'FAL1785', 'FAL1910',
                             'memo4'],
                             'Kompetenzen', vpf)
    komp = show_first_row(komp)
    xmlw.add_element('problfam_a', map_problfam(komp[0]))
    xmlw.add_element('problfam_b', map_problfam(komp[1]))
    xmlw.add_element('problfam_d', map_problfam(komp[2]))
    xmlw.add_element('problfam_e', map_problfam(komp[3]))
    xmlw.add_element('problfam_f', map_problfam(komp[4]))
    xmlw.add_element('problfam_g', map_problfam(komp[5]))
    xmlw.add_element('problfam_h', map_problfam(komp[6]))
    xmlw.add_element('problfam_i', map_problfam(komp[7]))
    xmlw.add_element('problfam_j', map_problfam(komp[8]))
    xmlw.add_element('problfam_k', map_problfam(komp[9]))
    xmlw.add_element('problfam_l', map_problfam(komp[10]))
    xmlw.add_element('problfam_m', map_problfam(komp[11]))
    xmlw.add_element('problfam_m_t', komp[12])

    t = u''
    if komp[13] != None:
        t += to_unicode(komp[13]) + '; '
    if komp[14] != None:
        t += to_unicode(komp[14])
    xmlw.add_element('familie_anm', t)

    anmerkungen = []
    komp = mder.save_select(['FAL1920', 'FAL1925', 'FAL1790', 'FAL1800', 'FAL1810', 'FAL1820', 'FAL1830', 'FAL1840', 'FAL1850', 'FAL1860', 'FAL1870', 'FAL1880', 'FAL1890', 'FAL1895', 'memo5'], 'Kompetenzen', vpf)

    komp = show_first_row(komp)

    # Alle Angaben der DV werden in das Anmerkungsfeld geschrieben

    if komp[0] == 'ja':
        anmerkungen.append(u'HzE fr jungen Menschen in der Vergangenheit/Gegenwart: ja')
    elif komp[0] == 'nein':
        anmerkungen.append(u'HzE fr jungen Menschen in der Vergangenheit/Gegenwart: nein')
    else:
        anmerkungen.append(u'HzE fr jungen Menschen in der Vergangenheit/Gegenwart: unbekannt')
    anmerkungen.append(u'Art der HzE fr jungen Menschen in der Vergangenheit/Gegenwart: %s' % extract_all_strings(komp[1]))
    anmerkungen.append('')
    if komp[2] == 'ja':
        anmerkungen.append(u'HzE anderer Familienmitglieder in der Vergangenheit/Gegenwart: ja')
    elif komp[2] == 'nein':
        anmerkungen.append(u'HzE anderer Familienmitglieder in der Vergangenheit/Gegenwart: nein')
    else:
        anmerkungen.append(u'HzE anderer Familienmitglieder in der Vergangenheit/Gegenwart: unbekannt')

    hze_label = {'0': u'Erziehungsberatung (28)',
                 '1': u'Soziale Gruppenarbeit (29)',
                 '2': u'Erziehungsbeistand, Betreuungshelfer (30)',
                 '3': u'SPFH (31)',
                 '4': u'Erziehung in einer Tagesgruppe (32)',
                 '5': u'Vollzeitpflege (33)', 
                 '6': u'Heimerziehung (34)',
                 '7': u'Intensive Einzelbetreuung (35)',
                 '8': u'Jugendsozialarbeit (13)',
                 '9': u'Sonstiges'}

    anmerkungen.append('Art der HzE anderer Familienmitglieder:')
    for num, hze in enumerate(komp[3:-3]):
        label = hze_label.get(str(num))
        if hze == 'ja':
            anmerkungen.append('* %s: %s' % (label, 'ja'))
        elif hze == 'nein':
            anmerkungen.append('* %s: %s' % (label, 'nein'))
        else:
            anmerkungen.append('* %s: %s' % (label, 'unbekannt'))

    # Behandele Sonstiges Feld
    label = hze_label.get(str(9))
    if komp[12]== 'ja':
        anmerkungen.append('* %s: %s' % (label, extract_all_strings(komp[13] or 'ja, aber unbekannt')))
    elif komp[12] == 'nein':
        anmerkungen.append('* %s: %s' % (label, 'nein'))
    else:
        anmerkungen.append('* %s: %s' % (label, 'unbekannt'))

    # Notizen
    anmerkungen.append('')
    anmerkungen.append('Notizen:')
    anmerkungen.append(extract_all_strings(komp[14]))


    # Da Angaben nicht importiert werden knnen, setze alle auf unbekannt.
    xmlw.add_element('uleistung_beginn', '-3')
    xmlw.add_element('uleistung_beginn_a', '-3')
    xmlw.add_element('uleistung_beginn_b', '-3')
    xmlw.add_element('uleistung_beginn_c', '-3')
    xmlw.add_element('uleistung_beginn_d', '-3')
    xmlw.add_element('uleistung_beginn_e', '-3')
    xmlw.add_element('uleistung_beginn_f', '-3')
    xmlw.add_element('uleistung_beginn_g', '-3')
    xmlw.add_element('uleistung_beginn_h', '-3')
    xmlw.add_element('uleistung_beginn_i', '-3')
    xmlw.add_element('uleistung_beginn_j', '-3')
    xmlw.add_element('uleistung_verg', '-3')

    # Schreibe alle Daten in das Anmerknungsfeld
    xmlw.add_element('uleistung_anm', '\n'.join(anmerkungen))


    komp = mder.save_select(['memo3'], 'Kompetenzen', vpf)
    komp = extract_all_strings(komp)
    dok_entw_5_1 += u'Entwicklung des jungen Menschen:\n%s\n\n' % komp

    # Cm Ende
    cmend = mder.save_select(['FAL4060', 'FAL4110', 'FAL4070', 'FAL4080', 'FAL4090',
                              'FAL4100', 'FAL4065', 'FAL4150', 'FAL4160', 'FAL4190', 'FAL4200', 'FAL4130',
                              'FAL4135', 'FAL4140', 'FAL4145', 'memo6'], 'Fall', vpf)
    cmend = show_first_row(cmend)

    xmlw.add_element('cm_end_jg', cmend[1])
    xmlw.add_element('cm_schule_name', cmend[2])
    cm_snhn = sn_hn(cmend[3])
    #xmlw.add_element('cm_schule_strasse', split_str_nr(cmend, 3, 5)[0])
    #xmlw.add_element('cm_schule_strassenr', split_str_nr(cmend, 3, 5)[1])
    xmlw.add_element('cm_schule_strasse',   cm_snhn[0])
    xmlw.add_element('cm_schule_strassenr', cm_snhn[1])
    xmlw.add_element('cm_schule_plz', cmend[4])
    xmlw.add_element('cm_schule_ort', cmend[5])

    if extract_all_strings(cmend[13]) == 'ja':
        xmlw.add_element('dok_abschl', extract_all_strings(cmend[14])) # Archiv bennante UA 
    xmlw.add_element('dok_ziel', extract_all_strings(cmend[15])) # Memo

    # Kompetenzen
    komp = mder.save_select(['FAL1260', 'FAL1270', 'FAL1290', 'FAL1300', 'FAL1310',
                             'FAL1320', 'FAL1220', 'FAL1180', 'FAL1190', 'FAL1200',
                             'FAL1210', 'FAL1480', 'FAL1490', 'FAL1560', 'FAL1500',
                             'FAL1540', 'FAL1550', 'FAL1570', 'FAL1520', 'FAL1525',
                             'FAL1510', 'FAL1515', 'FAL1530', 'FAL1535', 'FAL1580',
                             'FAL1590'], 'Kompetenzen', vpf)
    komp = show_first_row(komp)
    xmlw.add_element('komp_schule_a_1', komp[0])
    xmlw.add_element('komp_schule_d_1', komp[1])
    xmlw.add_element('komp_deutsch_a_1', komp[2])
    xmlw.add_element('komp_deutsch_b_1', komp[3])
    xmlw.add_element('komp_deutsch_c_1', komp[4])
    xmlw.add_element('komp_deutsch_d_1', komp[5])
    xmlw.add_element('komp_sozial_c_1', komp[6])
    xmlw.add_element('komp_selbst_d_1', komp[7])
    xmlw.add_element('komp_psy_a_1', komp[8])
    xmlw.add_element('komp_psy_b_1', komp[9])
    xmlw.add_element('komp_psy_f_1', komp[10])
    xmlw.add_element('entw_a_1', map_problfam(komp[11]))
    xmlw.add_element('entw_b_1', map_problfam(komp[12]))
    xmlw.add_element('entw_c_1', map_problfam(komp[13]))
    xmlw.add_element('entw_d_1', map_problfam(komp[14]))
    xmlw.add_element('entw_e_1', map_problfam(komp[15]))
    xmlw.add_element('entw_f_1', map_problfam(komp[16]))
    xmlw.add_element('entw_g_1', map_problfam(komp[17]))
    xmlw.add_element('entw_h_1', map_problfam(komp[18]))
    xmlw.add_element('entw_h_t_1', komp[19])
    xmlw.add_element('entw_i_1', map_problfam(komp[20]))
    xmlw.add_element('entw_i_t_1', komp[21])
    xmlw.add_element('entw_j_1', map_problfam(komp[22]))
    xmlw.add_element('entw_j_t_1', komp[23])
    xmlw.add_element('entw_k_1', map_problfam(komp[24]))
    xmlw.add_element('entw_k_t_1', komp[25])

    # Beschulung

    beschul_end_mapping(vpf, mder, xmlw)

    beschul = mder.save_select(['FAL2010', 'FAL2030', 'FAL2035', 'FAL2040', 'FAL2050',
                                'FAL2060', 'memo1'], 'Bildungsplan', vpf)
    beschul = show_first_row(beschul)
    b_a = extract_all_strings(beschul[0])

    beschul_b = mder.save_select(['FAL3010', 'FAL3015'], 'Fallbegleitung', vpf, add='ORDER BY Datum DESC')
    beschul_b = show_first_row(beschul_b)
    b_b = extract_all_strings(beschul_b[0])

    if b_b != '' and b_b != b_a:
        b_a = b_b

    if (b_a == 'in der abgebenden Schule, in der Ursprungsklasse'
         or b_a == 'in der Herkunftsklasse'):
        xmlw.add_element('beschul_a', '1')
        #xmlw.add_element('beschul_end_a', '1')
    else:
        xmlw.add_element('beschul_a', '-1')
        #xmlw.add_element('beschul_end_a', '-1')

    if (b_a == u'in der abgebenden Schule, in einer gesonderten Lerngruppe'
         or b_a == u'innerhalb der Herkungsschule in einer extra Klasse'):
        xmlw.add_element('beschul_b', '1')
        #xmlw.add_element('beschul_end_b', '1')
    else:
        xmlw.add_element('beschul_b', '-1')
        #xmlw.add_element('beschul_end_b', '-1')

    if (b_a == u'an einem auerschulischen Lernort'
         or b_a == 'ausserschulischer Lernort'):
        xmlw.add_element('beschul_c', '1')
        #xmlw.add_element('beschul_end_c', '1')
    else:
        xmlw.add_element('beschul_c', '-1')
        #xmlw.add_element('beschul_end_c', '-1')

    if b_a == u'gem eines Phasenmodells auerhalb und innerhalb der Schule':
        xmlw.add_element('beschul_d', '1')
        #xmlw.add_element('beschul_end_d', '1')
    else:
        xmlw.add_element('beschul_d', '-1')
        #xmlw.add_element('beschul_end_d', '-1')

    if b_a == u'in einer anderen Schule':
        xmlw.add_element('beschul_e', '1')
        #xmlw.add_element('beschul_end_e', '1')
    else:
        xmlw.add_element('beschul_e', '-1')
        #xmlw.add_element('beschul_end_e', '-1')

    if b_a == u'Praktikum/Betrieb':
        xmlw.add_element('beschul_f', '1')
        #xmlw.add_element('beschul_end_f', '1')
    else:
        xmlw.add_element('beschul_f', '-1')
        #xmlw.add_element('beschul_end_f', '-1')

    if b_a == u'Sonstiges':
        xmlw.add_element('beschul_h', '1')
        #xmlw.add_element('beschul_end_h', '1')
        xmlw.add_element('beschul_h_t', beschul_b[1])
        #xmlw.add_element('beschul_end_h_t', beschul_b[1])
    else:
        xmlw.add_element('beschul_h', '-1')
        #xmlw.add_element('beschul_end_h', '-1')

    xmlw.add_element('reint_ziel', map_reint_ziel(beschul[1]))
    xmlw.add_element('reint_ziel_t', beschul[2])
    xmlw.add_element('reint_beding_a', beschul[3])
    xmlw.add_element('reint_beding_b', beschul[4])
    xmlw.add_element('reint_beding_c', beschul[5])
    xmlw.add_element('reint_beding_d', beschul[6])

    probl = mder.save_select(['FAL2070', 'FAL2080', 'FAL2090', 'FAL2100',
                              'FAL2110'], 'Bildungsplan', vpf)

    # TODO Issue 11. Probleme unter 5.2.2
    probl = show_first_row(probl)
    prob_text = extract_all_strings(probl[0])

    aenderungen = mder.save_select(['FAL3640', 'FAL3645'], 'Fallbegleitung', vpf,add='ORDER BY Datum DESC')
    aenderung = show_first_row(aenderungen)
    if extract_all_strings(aenderung[0]) == '-1':
        p_text = extract_all_strings(aenderung[1])
        if prob_text:
            prob_text += u'\n\n%s' % p_text 
        else:
            prob_text = p_text 

    xmlw.add_element('probleme_a', prob_text)

    xmlw.add_element('probleme_b', probl[1])
    xmlw.add_element('probleme_c', probl[2])
    xmlw.add_element('probleme_d', probl[3])
    xmlw.add_element('probleme_e', probl[4])

    problem_notiz = mder.save_select(['memo2'], 'Bildungsplan', vpf)
    problem_notiz = show_first_row(problem_notiz)
    if problem_notiz:
        xmlw.add_element('probleme_f', extract_all_strings(problem_notiz))



    cm_st = mder.save_select(['FAL4050', 'FAL4170', 'FAL4180'], 'Fall', vpf)
    cm_st = show_first_row(cm_st)

    # Bezugspersonen
    hilfex = mder.save_select(['fnr', 'Datum', 'hilfe_nr', 'FAL2120', 'FAL2130', 'FAL2140',
                               'FAL2150', 'FAL2160', 'FAL2170', 'FAL2180', 'FAL2190',
                               'FAL2200', 'memo'], 'Hilfe', vpf)

    basisfall = {}
    fallbegl = {}
    for h in hilfex:
        if h[1] != None and h[1] == '0':
            if not basisfall.has_key(h[3]):
                basisfall[h[3]] = h
            elif mk_utime(h[9]) > mk_utime(basisfall[h[3]][9]):
                basisfall[h[3]] = h

        else:
            if not fallbegl.has_key(h[3]):
                fallbegl[h[3]] = h
            elif int(h[1]) > int(fallbegl[h[3]][1]):
                fallbegl[h[3]] = h

    for h in basisfall.keys():
        if fallbegl.has_key(h):
            del basisfall[h]

    hilfe = []
    for h in fallbegl.keys():
        hilfe.append(fallbegl[h])

    for h in basisfall.keys():
        hilfe.append(basisfall[h])

    #ziel_er = mder.save_select(['FAL3955', 'FAL3965', 'FAL3975'], 'Fallbegleitung', vpf,
    #                            add='ORDER BY Datum DESC')

    #ziel_er = show_first_row(ziel_er)

    hilfe_plus = ''
    i = 1
    for e in range(0, len(hilfe)):

        he = hilfe[e]

        anmerkung = ''
        if he[12] != None:
            anmerkung = he[12]

        if i <= 3:
            found_hilfe = False
            found_hilfe |= xmlw.add_element('hilfm_art_%d' % i, he[3])
            found_hilfe |= xmlw.add_element('hilfm_inst_%d' % i, he[7])
            found_hilfe |= xmlw.add_element('hilfm_einr_%d' % i, he[8])
            found_hilfe |= xmlw.add_element('hilfm_start_%d' % i, transform_do(str(he[9])))
            found_hilfe |= xmlw.add_element('hilfm_ende_%d' % i, transform_do(str(he[10])))
            found_hilfe |= xmlw.add_element('hilfm_umf_%d' % i, he[11])
            found_hilfe |= xmlw.add_element('hilfm_ziel_%d_1' % i, he[4])
            #xmlw.add_element('hilfm_erreicht_%d_1' % i, map_ziel_er(ziel_er[0]))
            found_hilfe |= xmlw.add_element('hilfm_ziel_%d_2' % i, he[5])
            #xmlw.add_element('hilfm_erreicht_%d_2' % i, map_ziel_er(ziel_er[1]))
            found_hilfe |= xmlw.add_element('hilfm_ziel_%d_3' % i, he[6])
            # xmlw.add_element('hilfm_erreicht_%d_3' % i, map_ziel_er(ziel_er[2]))
            found_hilfe |= xmlw.add_element('hilfm_sf_%d' % i, anmerkung)
            if found_hilfe:
                xmlw.add_element('hilfm_%d' % i, '1')
                i += 1
        else:
            hilfe_plus += (u'Art der Hilfe: %s\n'
                           u'durchfhrende Institution/Person: %s\n'
                           u'Einrichtung/Dienst/Person: %s\n'
                           u'Datum Beginn der Hilfe: %s\n'
                           u'Datum Ende der Hilfe: %s\n'
                           u'Zeitlicher Umfang der Hilfe in Wochenstunden: %s\n'
                           u'Nennung Ziel 1: %s\n'
                           #u'Zielerreichung 1: %s\n'
                           u'Nennung Ziel 2: %s\n'
                           #u'Zielerreichung 2: %s\n'
                           u'Nennung Ziel 3: %s\n'
                           #u'Zielerreichung 3: %s\n'
                           u'Schlussfolgerung und Anmerkung: %s\n\n'
                           % (he[3], he[7], he[8], he[9], he[10],
                              he[11], he[4],
                              # map_ziel_er_t(ziel_er[0]), 
                              he[5],
                              # map_ziel_er_t(ziel_er[1]), 
                              he[6], 
                              # map_ziel_er_t(ziel_er[2]),
                              anmerkung))

    xmlw.add_element('hilfm_weitere', hilfe_plus)

    # Vereinbarungen, elternarbeit
    vereinb = mder.save_select(['FAL2430', 'FAL2440', 'FAL2450', 'FAL2460',
                            'FAL2478', 'FAL2479', 'FAL2480', 'FAL2481', 'FAL2482',
                            'FAL2483', 'FAL2484', 'FAL2485', 'FAL2490', 'FAL2500',
                            'FAL2510', 'FAL2520', 'FAL2530','FAL2470', 'memo3'], 'Bildungsplan', vpf)
    vereinb = show_first_row(vereinb)
    xmlw.add_element('vereinb_a',    vereinb[0])
    xmlw.add_element('vereinb_b',    vereinb[1])
    xmlw.add_element('vereinb_c',    vereinb[2])
    xmlw.add_element('vereinb_d',    vereinb[3])
    xmlw.add_element('vereinb_e',    vereinb[17])

    vereinb_f = extract_all_strings(vereinb[18])
    # xmlw.add_element('vereinb_f', vereinb_f)
    xmlw.add_element('elt_ziel_i_t', vereinb[11])


    elt_ziel = mder.save_select(['FAL2478', 'FAL2479', 'FAL2480', 'FAL2481', 'FAL2482',
                                 'FAL2483', 'FAL2484', 'FAL2485'], 'Bildungsplan', vpf)
    f_eziel = mder.save_select(['FAL3880', 'FAL3881', 'FAL3882', 'FAL3883', 'FAL3884', 'FAL3885',                                 'FAL3886', 'FAL3887'], 'Fallbegleitung', vpf, add='ORDER BY Datum DESC')

    elt_ziel = show_first_row(elt_ziel)
    for row in f_eziel:
        if extract_all_strings(row[0]) != '0':
            for num, field in enumerate(row[1:]):
                if field == '-1':
                    elt_ziel[num] = '-1'

    if elt_ziel[0] == '-1':
        xmlw.add_element('elt_ziel_a', '-3')
    if elt_ziel[1] == '-1':
        xmlw.add_element('elt_ziel_b', '-3')
    if elt_ziel[2] == '-1':
        xmlw.add_element('elt_ziel_c', '-3')
    if elt_ziel[3] == '-1':
        xmlw.add_element('elt_ziel_d', '-3')
    if elt_ziel[4] == '-1':
        xmlw.add_element('elt_ziel_e', '-3')
    if elt_ziel[5] == '-1':
        xmlw.add_element('elt_ziel_f', '-3')
    if elt_ziel[6] == '-1':
        xmlw.add_element('elt_ziel_g', '-3')
    elt_ziel[7] = extract_all_strings(elt_ziel[7])
    if elt_ziel[7] != '':
        xmlw.add_element('elt_ziel_i', '-3')
        xmlw.add_element('elt_ziel_i_t', elt_ziel[7])




    vereinb = mder.save_select(['FAL2490', 'FAL2500','FAL2510',
                                'FAL2520', 'FAL2530','FAL2535', 
                                'FAL2540','FAL2545'], 'Bildungsplan', vpf)
    vereinb = show_first_row(vereinb)

    elt_a = mder.save_select(['FAL3900', 'FAL3910', 'FAL3920', 
                              'FAL3930', 'FAL3940', 'FAL3945' ],
                              'Fallbegleitung', vpf)

    elt_mn_a = ''
    elt_mn_b = ''
    elt_mn_h = ''
    elt_mn_i = ''
    elt_mn_j = '' 
    elt_mn_j_t = []
    j_flag = 0

    # Fuer die Basisfallakte
    # Wert 6 = "Methode wird nicht angewandt". Werte sollen nicht uebernommen
    # werden. Geklaert in einem Telefonat zwischen Herrn Kuehle und Herrn
    # Irlaender (28.05.2009)
    try:
        if int(extract_all_strings(vereinb[0])) < 6:
            elt_mn_a = '1'
        else:
            elt_mn_a= '0'
    except ValueError:
        pass
    try:
        if int(extract_all_strings(vereinb[1])) < 6:
            elt_mn_b = '1'
        else:
            elt_mn_b= '0'
    except ValueError:
        pass
    try:
        if int(extract_all_strings(vereinb[2])) < 6:
            elt_mn_h = '1'
        else:
            elt_mn_h= '0'
    except ValueError:
        pass
    try:
        if int(extract_all_strings(vereinb[3])) < 6:
            elt_mn_i = '1'
        else:
            elt_mn_i = '0'
    except ValueError:
        pass
    try:
        if int(extract_all_strings(vereinb[5])) < 6:
            j_flag |= 2
            elt_mn_j_t.append(extract_all_strings(vereinb[4]))
        else:
            j_flag |= 1
    except ValueError:
        pass
    try:
        if int(extract_all_strings(vereinb[7])) < 6:
            j_flag |= 2
            elt_mn_j_t.append(extract_all_strings(vereinb[6]))
        else: 
            j_flag |= 1
    except ValueError:
        pass

    if j_flag in (2,3):
        elt_mn_j = '1'
    elif j_flag == 1:
        elt_mn_j = '0'

    # Fuer alle Fallbegleitungen
    for e in elt_a:
        #print extract_all_strings(e[0])
        if extract_all_strings(e[0]) not in ('','keine'): # kurze Einzelgespraeche
            elt_mn_a = '1'
        if extract_all_strings(e[1]) not in ('','keines'): # intensive Beratung 
            elt_mn_b = '1'
        if extract_all_strings(e[2]) not in ('','nein'): # Elternbildungsarbeit 
            elt_mn_h = '1'
        if extract_all_strings(e[3]) not in ('','nein'): # Einbezug der Eltern 
            elt_mn_i = '1'
        if extract_all_strings(e[4]) not in ('','nein'): # Sonstige 
            elt_mn_j = '1'
            elt_mn_j_t.append(extract_all_strings(e[5]))

    xmlw.add_element('elt_mn_a', elt_mn_a)
    xmlw.add_element('elt_mn_b', elt_mn_b)
    xmlw.add_element('elt_mn_h', elt_mn_h)
    xmlw.add_element('elt_mn_i', elt_mn_i)
    xmlw.add_element('elt_mn_j', elt_mn_j)
    xmlw.add_element('elt_mn_j_t', "; ".join(elt_mn_j_t))

    elt_notiz = mder.save_select(['memo3'], 'Fallbegleitung', vpf)

    xmlw.add_element('vereinb_f', u'%s\n%s' % (vereinb_f, extract_all_strings(elt_notiz)))

    #elt_notiz_1 = mder.save_select(['memo4'], 'Bildungsplan', vpf)
    #dok_cm_5_4 += u'Elternarbeit:\n%s\n\n' % extract_all_strings(elt_notiz_1)
    dok_cm_5_4 += u'Elternarbeit (Fallbegleitung D):\n%s\n\n' % extract_all_strings(elt_notiz)

    # Ende CM
    cm_st = mder.save_select(['FAL4150', 'FAL4160', 'FAL4170', 'FAL4180', 'FAL4190', 'FAL4200'], 'Fall', vpf)
    cm_st = show_first_row(cm_st)

    xmlw.add_element('cm_zielsch_a', map_zielsch(cm_st[0]))
    xmlw.add_element('cm_zielsch_b', map_zielsch(cm_st[1]))
    xmlw.add_element('cm_zielsch_d', map_zielsch(cm_st[2]))
    xmlw.add_element('cm_zielsoz_a', map_zielsch(cm_st[3]))
    xmlw.add_element('cm_zielsoz_b', map_zielsch(cm_st[4]))
    xmlw.add_element('cm_zielsch_g', map_zielsch(cm_st[5]))

    if transform_do(cm_st[0]) != '':
        xmlw.add_element('datum_cm_ende', transform_do(cm_st[0]))

    # Nachbetreuung
    nachb = mder.save_select(['FAL4130', 'FAL4135'], 'Fall', vpf)
    nachb = show_first_row(nachb)
    xmlw.add_element('nbetr_verbl', '-1')
    xmlw.add_element('nbetr_verbl_t', nachb[1])
    xmlw.add_element('dok_cm', dok_cm_5_4)
    xmlw.add_element('dok_angaben', dok_angaben_3_7)
    xmlw.add_element('dok_entw', dok_entw_5_1)

    # TODO: Was ist mit Notiz (memo2)?
    zugeh = mder.save_select(['FAL1440', 'FAL1460', 'FAL1470', 'FAL1450', 'memo2'], 'Kompetenzen', vpf)
    zugeh = show_first_row(zugeh)

    xmlw.add_element('zugeh_a', map_zugeh(zugeh[0]))
    xmlw.add_element('zugeh_b', map_zugeh(zugeh[1]))
    xmlw.add_element('zugeh_c', map_zugeh(zugeh[2]))
    xmlw.add_element('zugeh_d', map_zugeh(zugeh[3]))

    komp_anm = extract_all_strings(zugeh[4])
    if komp_anm:
        xmlw.add_element('komp_anm', komp_anm)

    cm_end = mder.save_select(['FAL4060', 'FAL4120', 'FAL4125', 'FAL4130', 'FAL4135'],
                               'Fall', vpf)
    cm_end = show_first_row(cm_end)
    xmlw.add_element('cm_end_art', map_cm_end_art(cm_end[0]))
    xmlw.add_element('cm_end_verm', map_cm_end_verm(cm_end[1]))
    xmlw.add_element('cm_end_verm_t', cm_end[2])

    if cm_end[3] != None:
        if cm_end[3] == 'Krankenhaus':
            xmlw.add_element('cm_end_sonst', '3')
        elif cm_end[3] == 'Ausbildung/Arbeit':
            xmlw.add_element('cm_end_sonst', '6')
        elif cm_end[3] == 'Haft':
            xmlw.add_element('cm_end_sonst', '5')
        else:
            xmlw.add_element('cm_end_sonst', '-2')
            if cm_end[3] != 'Sonstiges':
                xmlw.add_element('cm_end_sonst_t', cm_end[3])
            else:
                xmlw.add_element('cm_end_sonst_t', cm_end[4])

    if map_cm_end_art(cm_end[0]) == '3':
        xmlw.add_element('cm_end_ab', '1')
        xmlw.add_element('cm_end_abgr', '-3')

